@@ -27,7 +27,6 @@ from gettext import gettext as _
 import gobject
 import gst
 import gnomevfs
-import gtk # TODO
 import gconf
 
 from fileoperations import vfs_encode_filename, file_encode_filename
@@ -40,7 +39,12 @@ from utils import debug, log
 from settings import mime_whitelist, filename_blacklist
 from error import SoundConverterException
 from error import show_error
-from notify import notification
+try:
+    from notify import notification
+except:
+    def notification(msg):
+        pass
+        
 from fnmatch import fnmatch
 
 import gconf
@@ -105,6 +109,7 @@ encoders = (
     ('lame',      'MP3'),
     ('faac',      'AAC'),
     ('mp4mux',    'AAC'),
+    ('opusenc',   'Opus'),
     )
 
 available_elements = set()
@@ -182,12 +187,7 @@ class Pipeline(BackgroundTask):
             return
         self.done()
         if result == gst.pbutils.INSTALL_PLUGINS_USER_ABORT:
-            dialog = gtk.MessageDialog(parent=None, flags=gtk.DIALOG_MODAL,
-                type=gtk.MESSAGE_INFO,
-                buttons=gtk.BUTTONS_OK,
-                message_format='Plugin installation aborted.')
-            dialog.run()
-            dialog.hide()
+            show_error(_('Plugin installation aborted.'))
             return
 
         show_error('Error', 'failed to install plugins: %s' % gobject.markup_escape_text(str(result)))
@@ -473,11 +473,13 @@ class Converter(Decoder):
 
         self.output_filename = output_filename
         self.output_type = output_type
-        self.vorbis_quality = None
-        self.aac_quality = None
-        self.mp3_bitrate = None
-        self.mp3_mode = None
-        self.mp3_quality = None
+        self.vorbis_quality = 0.6
+        self.aac_quality = 192
+        self.mp3_bitrate = 192
+        self.mp3_mode = 'vbr'
+        self.mp3_quality = 3
+        self.flac_compression = 8
+        self.wav_sample_width = 16
 
         self.output_resample = output_resample
         self.resample_rate = resample_rate
@@ -495,6 +497,7 @@ class Converter(Decoder):
             'audio/x-wav': self.add_wav_encoder,
             'audio/mpeg': self.add_mp3_encoder,
             'audio/x-m4a': self.add_aac_encoder,
+            'audio/ogg; codecs=opus': self.add_opus_encoder,
             'gst-profile': self.add_audio_profile,
         }
         self.add_command('audiorate')
@@ -515,13 +518,8 @@ class Converter(Decoder):
         if not encoder:
             # TODO: is this used ?
             # TODO: add proper error management when an encoder cannot be created
-            dialog = gtk.MessageDialog(None, gtk.DIALOG_MODAL,
-                        gtk.MESSAGE_ERROR,
-                        gtk.BUTTONS_OK,
-                        _("Cannot create a decoder for \'%s\' format.") % \
+            show_error(_("Cannot create a decoder for \'%s\' format.") % 
                         self.output_type)
-            dialog.run()
-            dialog.hide()
             return
 
         self.add_command(encoder)
@@ -531,14 +529,7 @@ class Converter(Decoder):
         if dirname and not gnomevfs.exists(dirname):
             log('Creating folder: \'%s\'' % dirname)
             if not vfs_makedirs(str(dirname)):
-                # TODO add better error management
-                dialog = gtk.MessageDialog(None, gtk.DIALOG_MODAL,
-                            gtk.MESSAGE_ERROR,
-                            gtk.BUTTONS_OK,
-                            _("Cannot create \'%s\' folder.") % \
-                            dirname)
-                dialog.run()
-                dialog.hide()
+                show_error(_("Cannot create \'%s\' folder.") % dirname)
                 return
 
         self.add_command('%s location="%s"' % (
@@ -588,6 +579,9 @@ class Converter(Decoder):
     def set_aac_quality(self, quality):
         self.aac_quality = quality
 
+    def set_opus_quality(self, quality):
+        self.opus_quality = quality
+
     def set_mp3_mode(self, mode):
         self.mp3_mode = mode
 
@@ -638,7 +632,8 @@ class Converter(Decoder):
             elif properties[self.mp3_mode][0]:
                 cmd += 'vbr-max-bitrate=320 '
             cmd += '%s=%s ' % (properties[self.mp3_mode][1], self.mp3_quality)
-            #cmd += 'lowpass-freq=22000 '
+            if properties[self.mp3_mode][0] and self.mp3_quality < 4:
+                cmd += 'lowpass-freq=20000 '
 
             if 'xingmux' in available_elements and properties[self.mp3_mode][0]:
                 # add xing header when creating VBR mp3
@@ -651,7 +646,10 @@ class Converter(Decoder):
         return cmd
 
     def add_aac_encoder(self):
-        return 'faac profile=2 bitrate=%s ! mp4mux' % (self.aac_quality * 1000)
+        return 'faac bitrate=%s ! mp4mux' % (self.aac_quality * 1000)
+
+    def add_opus_encoder(self):
+        return 'opusenc bitrate=%s ! oggmux' % (self.opus_quality * 1000)
 
     def add_audio_profile(self):
         pipeline = audio_profiles_dict[self.audio_profile][2]
@@ -732,7 +730,7 @@ class ConverterQueue(TaskQueue):
                 dpath = gobject.markup_escape_text(dpath)
 
                 msg = \
-                _('The output file <i>%s</i>\n exists already.\n '\
+                _('The output file <i>%s</i>\n already exists.\n '\
                     'Do you want to skip the file, overwrite it or'\
                     ' cancel the conversion?\n') % dpath
 
@@ -773,6 +771,7 @@ class ConverterQueue(TaskQueue):
                         )
         c.set_vorbis_quality(self.window.prefs.get_float('vorbis-quality'))
         c.set_aac_quality(self.window.prefs.get_int('aac-quality'))
+        c.set_opus_quality(self.window.prefs.get_int('opus-bitrate'))
         c.set_flac_compression(self.window.prefs.get_int('flac-compression'))
         c.set_wav_sample_width(self.window.prefs.get_int('wav-sample-width'))
         c.set_audio_profile(self.window.prefs.get_string('audio-profile'))
@@ -817,7 +816,6 @@ class ConverterQueue(TaskQueue):
                 per_file_progress[task.sound_file] = taskprogress
         for task in self.waiting_tasks:
             prolist.append(0.0)
-            per_file_progress[task.sound_file] = 0.0
 
         progress = sum(prolist)/len(prolist) if prolist else 0
         progress = min(max(progress, 0.0), 1.0)
@@ -838,7 +836,7 @@ class ConverterQueue(TaskQueue):
         self.window.set_sensitive()
         self.window.conversion_ended()
         total_time = self.run_finish_time - self.run_start_time
-        msg = _('Conversion done, in %s') % self.format_time(total_time)
+        msg = _('Conversion done in %s') % self.format_time(total_time)
         if self.error_count:
             msg += ', %d error(s)' % self.error_count
         self.window.set_status(msg)
