@@ -201,6 +201,8 @@ class FileList:
         files = []
         self.window.set_status(_('Scanning files...'))
 
+        base = None
+
         for uri in uris:
             if not uri:
                 continue
@@ -229,6 +231,10 @@ class FileList:
 
             if info.type == gnomevfs.FILE_TYPE_DIRECTORY:
                 log('walking: \'%s\'' % uri)
+                if len(uris) == 1:
+                    # if only one folder is passed to the function,
+                    # use its parent as base path.
+                    base = os.path.dirname(uri)
                 filelist = vfs_walk(gnomevfs.URI(uri))
                 accepted = []
                 if extensions:
@@ -241,7 +247,8 @@ class FileList:
             else:
                 files.append(uri)
 
-        base, notused = os.path.split(os.path.commonprefix(files))
+        if not base:
+            base, notused = os.path.split(os.path.commonprefix(files))
         base += '/'
 
         for f in files:
@@ -279,8 +286,9 @@ class FileList:
 
     def set_row_progress(self, number, progress=None, text=None):
         self.progress_column.set_visible(True)
-
         if progress is not None:
+            if self.model[number][2] == 1.0:
+                return # already...
             self.model[number][2] = progress * 100.0
         if text is not None:
             self.model[number][3] = text
@@ -318,7 +326,7 @@ class FileList:
         return True
 
     def _append_file(self, sound_file):
-        self.model.append([self.format_cell(sound_file), sound_file, 0, '',
+        self.model.append([self.format_cell(sound_file), sound_file, 0.0, '',
                            sound_file.uri])
         self.filelist.add(sound_file.uri)
         sound_file.filelist_row = len(self.model) - 1
@@ -398,12 +406,13 @@ class PreferencesDialog(GladeWindow, GConfStore):
         'mp3-abr-quality': 192,
         'mp3-vbr-quality': 3,
         'aac-quality': 192,
+        'opus-bitrate': 96,
         'flac-compression': 8,
         'wav-sample-width': 16,
         'delete-original': 0,
         'output-resample': 0,
         'resample-rate': 48000,
-        'flac-speed': 0,
+        'flac-speed': 0, # TODO used ?
         'force-mono': 0,
         'last-used-folder': None,
         'audio-profile': None,
@@ -493,6 +502,7 @@ class PreferencesDialog(GladeWindow, GConfStore):
                     ('audio/x-flac'  , 'flacenc'),
                     ('audio/x-wav'   , 'wavenc'),
                     ('audio/x-m4a'   , 'faac'),
+                    ('audio/ogg; codecs=opus'   , 'opusenc'),
                     ('gst-profile'   , None),
                     ) # must be in same order in output_mime_type
 
@@ -564,6 +574,11 @@ class PreferencesDialog(GladeWindow, GConfStore):
         quality_setting = {64: 0, 96: 1, 128: 2, 192: 3, 256: 4, 320: 5}
         w.set_active(quality_setting.get(quality, -1))
 
+        w = self.opus_quality
+        quality = self.get_int('opus-bitrate')
+        quality_setting = {48: 0, 64: 1, 96: 2, 128: 3, 160: 4, 192: 5}
+        w.set_active(quality_setting.get(quality, -1))
+
         w = self.flac_compression
         quality = self.get_int('flac-compression')
         quality_setting = {0: 0, 5: 1, 8: 2}
@@ -639,6 +654,9 @@ class PreferencesDialog(GladeWindow, GConfStore):
         elif mime_type == 'audio/x-m4a':
             bitrate = self.get_int('aac-quality')
 
+        elif mime_type == 'audio/ogg; codecs=opus':
+            bitrate = self.get_int('opus-bitrate')
+
         elif mime_type == 'audio/mpeg':
             quality = {
                 'cbr': 'mp3-cbr-quality',
@@ -712,6 +730,7 @@ class PreferencesDialog(GladeWindow, GConfStore):
                         'audio/x-wav': '.wav',
                         'audio/mpeg': '.mp3',
                         'audio/x-m4a': '.m4a',
+                        'audio/ogg; codecs=opus': '.opus',
                         'gst-profile': '.' + profile_ext,
                     }.get(output_type, '.?')
 
@@ -856,7 +875,8 @@ class PreferencesDialog(GladeWindow, GConfStore):
                         'audio/x-flac': 2,
                         'audio/x-wav': 3,
                         'audio/x-m4a': 4,
-                        'gst-profile': 5,
+                        'audio/ogg; codecs=opus': 5,
+                        'gst-profile': 6,
         }
         self.quality_tabs.set_current_page(tabs[mime_type])
 
@@ -885,6 +905,10 @@ class PreferencesDialog(GladeWindow, GConfStore):
         if button.get_active():
             self.change_mime_type('audio/x-m4a')
 
+    def on_output_mime_type_opus_toggled(self, button):
+        if button.get_active():
+            self.change_mime_type('audio/ogg; codecs=opus')
+
     def on_vorbis_quality_changed(self, combobox):
         if combobox.get_active() == -1:
             return # just de-selectionning
@@ -911,6 +935,11 @@ class PreferencesDialog(GladeWindow, GConfStore):
         self.set_int('aac-quality', quality[combobox.get_active()])
         self.update_example()
 
+    def on_opus_quality_changed(self, combobox):
+        quality = (48, 64, 96, 128, 160, 192)
+        self.set_int('opus-bitrate', quality[combobox.get_active()])
+        self.update_example()
+
     def on_wav_sample_width_changed(self, combobox):
         quality = (8, 16, 32)
         self.set_int('wav-sample-width', quality[combobox.get_active()])
@@ -1249,8 +1278,9 @@ class SoundConverterWindow(GladeWindow):
                 extensions = [os.path.splitext(p)[1] for p in patterns]
             self.filelist.add_uris(folders, extensions=extensions)
 
-            self.prefs.set_string('last-used-folder',
-                            self.addfolderchooser.get_current_folder_uri())
+            folder = self.addfolderchooser.get_current_folder_uri()
+            if folder:
+                self.prefs.set_string('last-used-folder', folder)
 
         self.set_sensitive()
 
@@ -1282,7 +1312,7 @@ class SoundConverterWindow(GladeWindow):
         self.converter.add(sound_file)
 
     def on_progress(self):
-        if self.pulse_progress > 0: # still waiting for tags
+        if self.pulse_progress >= 0: # still waiting for tags
             self.set_progress(self.pulse_progress, display_time=False)
             return True
         if self.pulse_progress == -1: # still waiting for add
@@ -1291,14 +1321,18 @@ class SoundConverterWindow(GladeWindow):
 
         perfile = {}
         for s in self.filelist.get_files():
-            perfile[s] = 1.0
+            perfile[s] = None
         running, progress = self.converter.get_progress(perfile)
 
         if running:
             self.set_progress(progress)
             for sound_file, taskprogress in perfile.iteritems():
-                self.set_file_progress(sound_file, taskprogress)
-
+                if taskprogress > 0.0:
+                    sound_file.progress = taskprogress
+                    self.set_file_progress(sound_file, taskprogress)
+                if taskprogress is None and sound_file.progress:
+                    self.set_file_progress(sound_file, 1.0)
+                    sound_file.progress = None
         return running
 
     def do_convert(self):
@@ -1307,13 +1341,16 @@ class SoundConverterWindow(GladeWindow):
             gobject.timeout_add(100, self.on_progress)
             if self.prefs.require_tags:
                 self.progressbar.set_text(_('Reading tags...'))
+            else:
+                self.progressbar.set_text(_('Preparing conversion...'))
             files = self.filelist.get_files()
             total = len(files)
             for i, sound_file in enumerate(files):
                 gtk_iteration()
+                self.pulse_progress = float(i)/total
+                sound_file.progress = None
                 if self.prefs.require_tags:
                     self.read_tags(sound_file)
-                    self.pulse_progress = float(i)/total
                 else:
                     self.converter.add(sound_file)
 
@@ -1477,6 +1514,7 @@ def gui_main(name, version, gladefile, input_files):
     win = SoundConverterWindow(builder)
     import error
     error.set_error_handler(ErrorDialog(builder))
+    
     #error_dialog = MsgAreaErrorDialog(builder)
     #error_dialog.msg_area = win.msg_area
     #error.set_error_handler(error_dialog)
